/*
 * Move.java
 *
 * Created on 6. November 2007, 20:46
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

/**
 * 
 * A move in a chess game
 * @author Georg-Johann Lay
 */
class Move 
        extends java.util.Vector
        implements BOARD_CONST
{
    static final String STR_KILLS = ":";
    static final String STR_MOVES = "-";
    private static final String STR_EP    = " e.p.";
    private static final String STR_CHECK = " +";
    static final String STR_0_0   = "0-0";
    static final String STR_0_0_0 = "0-0-0";

    //private Board board;
    //int created = 0;
    Field f_from;
    Field f_to;

    int color;
    boolean isCheck;
    boolean isPawnMorph = false;

    private String str = null;
    private String str_from;
    private String str_to;
    private String str_ep = "";
    private String str_create = "";

    private boolean isKill;
    private int morphTo;

    private int lastPawn2x = -1;
    private int old_lastPawn2x;
    private int state;
    private int old_state;

    int rating = 0;
    
    static final String[] RATING = 
    {
        "??", "?", "?!", "" /* 0 */, "!?", "!", "!!"
    };

    Move (Board board, Field f_from, Field f_to, int morphTo)
    {
        super ();

        int x0,y0, x1,y1;

        //this.board = board;
        this.morphTo = morphTo;

        this.f_from = f_from;
        this.f_to   = f_to;

        int id  = board.p (x0=f_from.x, y0=f_from.y);
        int old = board.p (x1=f_to.x,   y1=f_to.y);

        color = board.c (x0,y0);

        old_lastPawn2x    = board.lastPawn2x;
        old_state = state = board.mayCastle;

        move (board, x0, y0, x1, y1);

        // must be fixed for e.p. later
        isKill = old != EMPTY;

        str_from = board.getString (x0, y0);
        str_to   = ""+f_to;

        int base = (color == WHITE) ? 0 : 7;

        if (ROOK == id)
        {
            if (y0 == base && x0 == 0)    state &= ~Board.M_0_0_0[color];
            if (y0 == base && x0 == 7)    state &= ~Board.M_0_0[color];
        }

        if (KING == id)
        {
            state &= ~Board.M_0_0[color];
            state &= ~Board.M_0_0_0[color];

            if (x1 == x0+2) // 0-0
            {
                move (board, 7, y0, 5, y0);
                str = STR_0_0;
            }

            if (x1 == x0-2) // 0-0-0
            {
                move (board, 0, y0, 3, y0);
                str = STR_0_0_0;
            }
        }

        if (PAWN == id)
        {
            if (x0 == x1  &&  2 == Math.abs (y0-y1))
                lastPawn2x = x0;

            // pawn takes pawn e.p.
            if (x0 != x1  &&  EMPTY == old)
            {
                    //System.err.println ("REMOVE.e.p "+y0+","+x1);
                    remove (board, x1,y0);
                    str_ep = STR_EP;

                    isKill = true;
            }

            if (y1 == 7-base)
            {
                isPawnMorph = true;

                addSet1 (board, x1, y1, morphTo);
                str_create = "=" + CHAR[morphTo >> 1];
            }
        }
    }

    private int move (Board board, int x0, int y0, int x1, int y1)
        {
            int pp = board.p[x1][y1];
            addSet1 (board, x1, y1, board.p[x0][y0]);
            remove (board, x0, y0);

            return pp;
        }

    private int remove (Board board, int x, int y)
    {
        int pp = board.p[x][y];

        if (pp != EMPTY)
            addSet1 (board, x, y, EMPTY);

        return pp;
    }

    private void addSet1 (Board board, int x, int y, int pp)
    {
        addElement (new Set1 (board, x, y, pp));
    }

    void apply (Board board)
    {
        for (int i = 0; i < size(); i++)
                ((Set1) elementAt(i)).apply(board);

        board.lastPawn2x = lastPawn2x;
        board.mayCastle      = state;
        board.mover ^= 1;
    }

    void undo (Board board)
    {
        for (int i = size()-1; i >= 0; i--)
                ((Set1) elementAt(i)).undo(board);

        board.lastPawn2x = old_lastPawn2x;
        board.mayCastle      = old_state;
        board.mover ^= 1;
    }

//#if CONSOLE
//#     public String toString ()
//#     {
//#         if (null == str)
//#             str = str_from
//#                     + ((isKill) ? STR_KILLS : STR_MOVES)
//#                     + str_to
//#                     + str_ep
//#                     + str_create;
//# 
//#         return str + ((isCheck) ? STR_CHECK : "");
//#     }
//#endif

//#if CONSOLE
//#     public String toShortString()
//#     {
//#         return "" + f_from + f_to 
//#                 + (isPawnMorph ? ""+CHAR[morphTo >> 1] : "");
//#     }
//#endif    
    
    // 16 bit
    // [ 2: 0] = f_from.x
    // [ 5: 3] = f_from.y
    // [ 8: 6] = f_to.x
    // [11: 9] = isPawnMorph ? morphTo/2 : f_to.y
    // [14:12] = rating+3
    short encode ()
    {
        int code = f_from.encode() | (f_to.encode() << 6);
        
        if (isPawnMorph)
            code = code & ((1<<9) -1) | ((morphTo >> 1) << 9);

        code |= (rating+3) << 12;
        return (short) code;
    }

    static Move Decode (Board b, short scode)
    {
        int color = b.mover;
        Field from = Field.Decode (scode & 0x3f);
        Field to   = Field.Decode ((scode>>6) & 0x3f);

        int morph  = 0;
        if (from.y == (WHITE == color ? 6 : 1)
            && PAWN == b.p(from.x, from.y))
        {
            morph = (to.y << 1) | color;
            to   = Field.Get (to.x, WHITE == color ? 7 : 0);
        }

        Move m = new Move (b, from, to, morph);
        
        m.rating = (7 & (scode >> 12)) -3;
        
        return m;
    }

    static Move FromString (Board b, String move)
    {
        Field from = Field.Get (move.substring (0, 2));
        Field to   = Field.Get (move.substring (2, 4));

        int morph = 0;

        if (move.length() >= 5
                && move.charAt(4) != ' ')
        {
            char c = move.charAt (4);
            for (int i = KNIGHT; i <= QUEEN; i+= 2)
                if (c == CHAR[i >> 1]) morph = i | b.mover;
        }
        
        int rate = 0;
        int r = move.indexOf (' ', 4);
        
        if (r > 0)
        {
            String s = move.substring (r+1) + "  ";
            rate  = s.charAt(0) == '?' ? -2 : s.charAt(0) == '!' ? 2 : 0;
            rate += s.charAt(1) == '?' ? -1 : s.charAt(1) == '!' ? 1 : 0;
        }
            

        Move m = new Move (b, from, to, morph);
        m.rating = rate;
        return m;
    }
    ////////////////////////////////////////////////////////////////////////


    class Set1
    {
        private int x, y;
        private int p_old, p_new;

        Set1 (Board b, int x, int y, int pp)
        {
            this.x = x;
            this.y = y;

            p_old = b.p[x][y];
            p_new = pp;
        }

        void apply (Board b)
        {
            b.p[x][y] = p_new;
            b.setKing (x,y);
        }

        void undo (Board b)
        {
            b.p[x][y] = p_old;
            b.setKing (x,y);
        }

        public String toString ()
        {
            return "["+x+","+y+"] "+p_old+"->"+p_new;
        }
    }
}

