/*
 * StringTools.java
 *
 * Created on 4. Dezember 2007, 17:02
 */
/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
 http://www.mozilla.org/MPL/
 
 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.
 
 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).
 
 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.
 
 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.
 
 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
 */

package de.gjlay.mobile.tools;

import java.io.IOException;
import java.io.InputStreamReader;
        
/**
 * Toolbox of utilities for file access
 * @author Georg-Johann Lay
 */
public class StringTools
{
    /**
     * Reads a single line from InputStreamReader
     * @param in InputStreamReader used to read the line
     * @throws IOException if there is any problem with reading
     * @return the read line
     */
    public static String readLine (InputStreamReader in)
        throws IOException
    {
        StringBuffer strBuf = new StringBuffer ("");
        int i;
        while ((i = in.read ()) != -1)
        {
            if ((char) i == '\r' || (char) i == '\n')
                return strBuf.toString ();
            strBuf.append ((char) i);
        }
        return strBuf.length () > 0 ? strBuf.toString () : null;
    }

    /**
     * Determines whether the line of the supplied string continues on the next line
     * @param line a line of String
     * @return true if the string contines on the next line, false otherwise
     */
    public static boolean continueLine (String line)
    {
        int slashCount = 0;
        int index = line.length () - 1;
        while ((index >= 0) && (line.charAt (index--) == '\\'))
            slashCount++;
        return (slashCount % 2 == 1);
    }
    
    /**
     * Decodes a string which uses unicode characters in \\uXXXX format.
     * @param theString String with \\uXXXX characters
     * @return resolved string
     */
    public static String convertString (String theString)
    {
        char aChar;
        int len = theString.length ();
        StringBuffer outBuffer = new StringBuffer (len);

        for (int x = 0; x < len;)
        {
            aChar = theString.charAt (x++);
            if (aChar == '\\')
            {
                aChar = theString.charAt (x++);
                if (aChar == 'u')
                {
                    // Read the xxxx
                    int value = 0;
                    for (int i = 0; i < 4; i++)
                    {
                        aChar = theString.charAt (x++);
                        switch (aChar)
                        {
                            case '0' :
                            case '1' :
                            case '2' :
                            case '3' :
                            case '4' :
                            case '5' :
                            case '6' :
                            case '7' :
                            case '8' :
                            case '9' :
                                value = (value << 4) + aChar - '0';
                                break;
                            case 'a' :
                            case 'b' :
                            case 'c' :
                            case 'd' :
                            case 'e' :
                            case 'f' :
                                value = (value << 4) + 10 + aChar - 'a';
                                break;
                            case 'A' :
                            case 'B' :
                            case 'C' :
                            case 'D' :
                            case 'E' :
                            case 'F' :
                                value = (value << 4) + 10 + aChar - 'A';
                                break;
                            default :
                                // return DEFAULT STRING if there is any problem
                                return "???";
                        }
                    }
                    outBuffer.append ((char) value);
                }
                else
                {
                    if (aChar == 't')
                        aChar = '\t';
                    else if (aChar == 'r')
                        aChar = '\r';
                    else if (aChar == 'n')
                        aChar = '\n';
                    else if (aChar == 'f')
                        aChar = '\f';
                    outBuffer.append (aChar);
                }
            }
            else
            {
                outBuffer.append (aChar);
            }
        }
        return outBuffer.toString ();
    }
}
