/*
 * Message.java
 *
 * Created on 28. November 2007, 15:07
 *
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.mobile.ui;

import de.gjlay.mobile.localize.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 * Shows a message
 * @author Georg-Johann Lay
 */
public class Message
        extends Alert
        implements CommandListener
{
    private MIDlet midlet;
    private static Message instance;

    public static final int ERROR   = 1;
    public static final int ERROR_EVENT   = 2;
    public static final int CONFIRM = 3;
    public static final int INFO = 4;

    private Command okCommand = null;
    private Command cancelCommand = null;
    private MessageListener listener;

    private int type;
    private int tag;

    private Message (MIDlet midlet)
    {
        super ("");
        this.midlet = midlet;
    }

    public static Message GetInstance (MIDlet midlet)
    {
        return
            instance = new Message (midlet);
    }

    private void init()
    {
    }

    public static void Show (MessageListener listener, int tag, int type, String message)
    {
        instance.listener = listener;
        instance.tag = tag;
        instance.type = type;
        instance.setString (message);
        instance.show();
    }

    private void show()
    {
        setTitle (L._("T_TITLE"));

        removeCommand (okCommand);
        removeCommand (cancelCommand);
        okCommand = cancelCommand = null;
        Command ok     = new Command (L._("T_OK"), Command.OK, 2);
        Command cancel = new Command (L._("T_BACK"), Command.CANCEL, 3);
        
        switch (type)
        {
            case ERROR:
                setType (AlertType.ERROR);
                setCommandListener (null);
                break;

            case INFO:
                setType (AlertType.INFO);
                setCommandListener (null);
                break;

            case ERROR_EVENT:
                setType (AlertType.ERROR);
                addCommand (okCommand = ok);
                setCommandListener (this);
                break;

            case CONFIRM:
                setType (AlertType.CONFIRMATION);
                addCommand (okCommand = ok);
                addCommand (cancelCommand = cancel);
                setCommandListener (this);
                break;
        }

        Display.getDisplay (midlet).setCurrent (this);
        // ChessLET.debug ("Message."+type);
    }

    public void commandAction (Command command, Displayable displayable)
    {
        listener.messageReturned (command.getCommandType(), tag);
    }

    public static void ShowInfo (String message)
    {
        Message.Show (new MessageListener()
            {
                /** unused dummy */
                public void messageReturned (int id, int tag) {}
            },
            0, INFO, message);
    }
}
