/*
 * Tiles.java
 *
 * Created on 18. November 2007, 20:47
 */
/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
 */

package de.gjlay.mobile.ui;

import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;

/**
 * Paints tiles onto a graphics object, similar to a TiledLayer. 
 * However, the tiles can be drawn on arbitrary positions and 
 * need not to be arranged in a grid.
 * @author Georg-Johann Lay
 * @see javax.microedition.lcdui.game.TiledLayer
 */
public class Tiles
{
    /**
     * Image containing the tiles
     */
    private Image image;
    /** tiles' width */
    int sizeX;

    /** tiles' height */
    int sizeY;

    /** tiles per row */
    private int tilesX;

    /**
     * Creates a new instance of Tiles
     * @param image contains the tiles.
     * image width must be a multiple of sizeX.
     * image height must be a multiple of sizeY.
     * All tiles have the same dimensions and are arranged in rows.
     * Enumeration starts at top left and preceeds to the right.
     * @param sizeX tiles' width
     * @param sizeY tiles' height
     */
    public Tiles (Image image, int sizeX, int sizeY)
    {
        this.image = image;
        this.sizeX = sizeX;
        this.sizeY = sizeY;
        tilesX = image.getWidth() / sizeX;
    }

    /**
     * Paint a tile on the graphics
     * @param g graphics object to paint the tile on
     * @param index tile's identifier. Tiles are enumerated linewise from
     *        top left (index=0) to bottom right.
     * @param x x-position
     * @param y y-position
     * @param flip draw upside down?
     * @param anchor anchor point
     * @see Graphics#drawImage(Image,int,int,int)
     */
    public void paint (Graphics g, int index, int x, int y, boolean flip, int anchor)
    {
        if (index >= 0)
        g.drawRegion (image,
                sizeX * (index % tilesX), sizeY * ((int) (index / tilesX)),
                sizeX, sizeY,
                (flip) ? Sprite.TRANS_ROT180 : Sprite.TRANS_NONE,
                x, y,
                anchor);
    }

    /**
     * Number of tiles per row
     * @return tiles per row
     */
    public int getTilesX()
    {
        return tilesX;
    }

    /**
     * Get tiles' width
     * @return tiles' width
     */
    public int getSizeX()
    {
        return sizeX;
    }

    /**
     * 
     * Get tiles'height
     * @return tiles' height
     */
    public int getSizeY()
    {
        return sizeY;
    }
}
